@@ -201,7 +201,8 @@ abstract class CControllerHost extends CController {
 			'objectids' => array_keys($triggers),
 			'source' => EVENT_SOURCE_TRIGGERS,
 			'object' => EVENT_OBJECT_TRIGGER,
-			'suppressed' => ($filter['show_suppressed'] == ZBX_PROBLEM_SUPPRESSED_TRUE) ? null : false
+			'suppressed' => ($filter['show_suppressed'] == ZBX_PROBLEM_SUPPRESSED_TRUE) ? null : false,
+			'symptom' => false
 		]);
 
 		$items = API::Item()->get([
