@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
  */
 
 $form = (new CForm())
-	->cleanItems()
 	->setId('preprocessing-test-form');
 
 if ($data['show_prev']) {
@@ -75,7 +74,7 @@ foreach ($data['macros'] as $macro_name => $macro_value) {
 				->removeAttribute('name')
 				->removeId()
 		))->addClass(ZBX_STYLE_TEXTAREA_FLEXIBLE_PARENT),
-		(new CCol('&rArr;'))->addStyle('vertical-align: top;'),
+		(new CCol(RARR()))->addStyle('vertical-align: top;'),
 		(new CCol(
 			(new CTextAreaFlexible('macros['.$macro_name.']', $macro_value))
 				->setWidth(ZBX_TEXTAREA_MACRO_VALUE_WIDTH)
@@ -86,7 +85,8 @@ foreach ($data['macros'] as $macro_name => $macro_value) {
 	]);
 }
 
-$form_grid = (new CFormGrid())->addClass(CFormGrid::ZBX_STYLE_FORM_GRID_3_1);
+$form_grid = (new CFormGrid())
+	->addClass(CFormGrid::ZBX_STYLE_FORM_GRID_3_1);
 
 if ($data['is_item_testable']) {
 	$form_grid->addItem([
@@ -150,6 +150,19 @@ if ($data['is_item_testable']) {
 				->addClass(CFormField::ZBX_STYLE_FORM_FIELD_FLUID)
 				->addClass('js-popup-row-snmp-community'),
 
+			(new CLabel(_('Max repetition count'), 'interface[details][max_repetitions]'))
+				->addClass('js-popup-row-snmp-max-repetition'),
+			(new CFormField(
+					(new CTextBox('interface[details][max_repetitions]',
+						$data['inputs']['interface']['details']['max_repetitions'], false,
+						CControllerPopupItemTest::INPUT_MAX_LENGTH
+					))
+						->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+						->setAriaRequired()
+				))
+					->addClass(CFormField::ZBX_STYLE_FORM_FIELD_FLUID)
+					->addClass('js-popup-row-snmp-max-repetition'),
+
 			(new CLabel(_('Context name'), 'interface[details][contextname]'))
 				->addClass('js-popup-row-snmpv3-contextname'),
 			(new CFormField(
@@ -273,7 +286,7 @@ $form_grid->addItem([
 			->setId('time')
 	),
 
-	($data['preproc_item'] instanceof CDiscoveryRule)
+	($data['test_type'] == CControllerPopupItemTestEdit::ZBX_TEST_TYPE_LLD)
 		? null
 		: (new CFormField((new CCheckBox('not_supported'))->setLabel(_('Not supported'))))
 			->addClass(CFormField::ZBX_STYLE_FORM_FIELD_FLUID),
@@ -366,9 +379,10 @@ if ($data['show_final_result']) {
 	]);
 }
 
-$form
-	->addItem($form_grid)
-	->addItem((new CInput('submit', 'submit'))->addStyle('display: none;'));
+$form->addItem($form_grid);
+
+// Enable form submitting on Enter.
+$form->addItem((new CSubmitButton(null))->addClass(ZBX_STYLE_FORM_SUBMIT_HIDDEN));
 
 $templates = [
 	(new CTag('script', true))
@@ -423,6 +437,7 @@ $warning_box = $data['show_warning']
 
 $output = [
 	'header' => $data['title'],
+	'doc_url' => CDocHelper::getUrl(CDocHelper::POPUP_TEST_EDIT),
 	'script_inline' => $this->readJsFile('popup.itemtestedit.view.js.php'),
 	'body' => (new CDiv([$warning_box, $form, $templates]))->toString(),
 	'cancel_action' => 'return saveItemTestInputs();',
