@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,9 +25,11 @@
 
 // Create form.
 $form = (new CForm())
+	->addItem((new CVar(CCsrfTokenHelper::CSRF_TOKEN_NAME, CCsrfTokenHelper::get(
+		$data['prototype'] ? 'itemprototype' : 'item'
+	)))->removeId())
 	->setId('massupdate-form')
 	->setName('massupdate-form')
-	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('ids', $data['ids'])
 	->addVar('action', $data['action'])
 	->addVar('prototype', $data['prototype'])
@@ -85,13 +87,12 @@ $item_form_list
 	->addRow(
 		(new CVisibilityBox('visible[post_type]', 'post_type_container', _('Original')))
 			->setLabel(_('Request body type')),
-		(new CDiv(
-			(new CRadioButtonList('post_type', (int) DB::getDefault('items', 'post_type')))
-				->addValue(_('Raw data'), ZBX_POSTTYPE_RAW)
-				->addValue(_('JSON data'), ZBX_POSTTYPE_JSON)
-				->addValue(_('XML data'), ZBX_POSTTYPE_XML)
-				->setModern(true)
-		))->setId('post_type_container')
+		(new CRadioButtonList('post_type', (int) DB::getDefault('items', 'post_type')))
+			->addValue(_('Raw data'), ZBX_POSTTYPE_RAW)
+			->addValue(_('JSON data'), ZBX_POSTTYPE_JSON)
+			->addValue(_('XML data'), ZBX_POSTTYPE_XML)
+			->setId('post_type_container')
+			->setModern(true)
 	)
 	->addRow(
 		(new CVisibilityBox('visible[timeout]', 'timeout', _('Original')))->setLabel(_('Timeout')),
@@ -127,7 +128,7 @@ $item_form_list
 				->addItem(new CRow([
 					(new CCol((new CDiv)->addClass(ZBX_STYLE_DRAG_ICON)))->addClass(ZBX_STYLE_TD_DRAG_ICON),
 					(new CTextBox('headers[name][#{index}]', '#{name}'))->setWidth(ZBX_TEXTAREA_HTTP_PAIR_NAME_WIDTH),
-					'&rArr;',
+					RARR(),
 					(new CTextBox('headers[value][#{index}]', '#{value}', false, 2000))
 						->setWidth(ZBX_TEXTAREA_HTTP_PAIR_VALUE_WIDTH),
 					(new CButton(null, _('Remove')))
@@ -204,17 +205,10 @@ $preprocessing_form_list = (new CFormList('preprocessing-form-list'))
 	->addRow(
 		(new CVisibilityBox('visible[preprocessing]', 'preprocessing_div', _('Original')))
 			->setLabel(_('Preprocessing steps')),
-		(new CDiv(getItemPreprocessing($form, [], false, $data['preprocessing_types'])))
+		(new CDiv(getItemPreprocessing([], false, $data['preprocessing_types'])))
 			->setId('preprocessing_div')
 	);
 
-// Prepare Update interval for form list.
-$update_interval = (new CTable())
-	->setId('update_interval')
-	->addRow([_('Delay'),
-		(new CDiv((new CTextBox('delay', ZBX_ITEM_DELAY_DEFAULT))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)))
-	]);
-
 $custom_intervals = (new CTable())
 	->setId('custom_intervals')
 	->setHeader([
@@ -267,19 +261,26 @@ $custom_intervals->addRow([(new CButton('interval_add', _('Add')))
 	->addClass(ZBX_STYLE_BTN_LINK)
 	->addClass('element-table-add')]);
 
-$update_interval->addRow(
-	(new CRow([
-		(new CCol(_('Custom intervals')))->addStyle('vertical-align: top;'),
-		new CCol($custom_intervals)
-	]))
-);
+// Prepare Update interval for form list.
+$update_interval = (new CTable())
+	->setId('update_interval')
+	->addRow([
+		_('Delay'),
+		new CDiv((new CTextBox('delay', ZBX_ITEM_DELAY_DEFAULT))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH))
+	])
+	->addRow(
+		(new CRow([
+			(new CCol(_('Custom intervals')))->addStyle('vertical-align: top;'),
+			new CCol($custom_intervals)
+		]))
+	);
 
 // Append update interval to form list.
 $item_form_list
 	// Append delay to form list.
 	->addRow(
-		(new CVisibilityBox('visible[delay]', 'update_interval_div', _('Original')))->setLabel(_('Update interval')),
-		(new CDiv($update_interval))->setId('update_interval_div')
+		(new CVisibilityBox('visible[delay]', 'update_interval', _('Original')))->setLabel(_('Update interval')),
+		$update_interval
 	)
 	// Append history to form list.
 	->addRow(
@@ -303,7 +304,7 @@ $item_form_list
 		(new CVisibilityBox('visible[trends]', 'trends_div', _('Original')))->setLabel(_('Trend storage period')),
 		(new CDiv([
 			(new CRadioButtonList('trends_mode', ITEM_STORAGE_CUSTOM))
-				->addValue(_('Do not keep trends'), ITEM_STORAGE_CUSTOM)
+				->addValue(_('Do not keep trends'), ITEM_STORAGE_OFF)
 				->addValue(_('Storage period'), ITEM_STORAGE_CUSTOM)
 				->setModern(true),
 			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -348,12 +349,12 @@ if ($data['single_host_selected'] && ($data['context'] === 'template' || !$data[
 		(new CDiv([
 			(new CMultiSelect([
 				'name' => 'valuemapid',
-				'object_name' => 'valuemaps',
+				'object_name' => $data['context'] === 'host' ? 'valuemaps' : 'template_valuemaps',
 				'multiple' => false,
 				'data' => [],
 				'popup' => [
 					'parameters' => [
-						'srctbl' => 'valuemaps',
+						'srctbl' => $data['context'] === 'host' ? 'valuemaps' : 'template_valuemaps',
 						'srcfld1' => 'valuemapid',
 						'dstfrm' => $form->getName(),
 						'dstfld1' => 'valuemapid',
@@ -362,8 +363,7 @@ if ($data['single_host_selected'] && ($data['context'] === 'template' || !$data[
 						'editable' => true
 					]
 				]
-			]))
-				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		]))->setId('valuemapid_div')
 	);
 }
@@ -480,7 +480,7 @@ $tags_form_list = (new CFormList('tags-form-list'))
 				->addStyle('margin-bottom: 10px;'),
 			renderTagTable([['tag' => '', 'value' => '']])
 				->setHeader([_('Name'), _('Value'), _('Action')])
-				->setId('tags-table')
+				->addClass('tags-table')
 		]))->setId('tags-div')
 	);
 
