@@ -60,7 +60,7 @@ $token_from_grid = (new CFormGrid())
 		new CFormField([
 			($data['expires_at'] == 0) ? '-' : date(DATE_TIME_FORMAT_SECONDS, (int) $data['expires_at']),
 			($data['expires_at'] != 0 && time() > $data['expires_at'])
-				? ['&nbsp;', makeErrorIcon(_('The token has expired. Please update the expiry date to use the token.'))]
+				? makeErrorIcon(_('The token has expired. Please update the expiry date to use the token.'))
 				: null
 		])
 	])
