@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -464,6 +464,10 @@ class CHost extends CHostGeneral {
 		if (is_array($options['filter'])) {
 			$this->dbFilter('hosts h', $options, $sqlParts);
 
+			if (array_key_exists('hostid', $options['filter'])) {
+				unset($options['filter']['hostid']);
+			}
+
 			if ($this->dbFilter('interface hi', $options, $sqlParts)) {
 				$sqlParts['left_join']['interface'] = ['alias' => 'hi', 'table' => 'interface', 'using' => 'hostid'];
 				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
@@ -503,7 +507,7 @@ class CHost extends CHostGeneral {
 		/*
 		 * Cleaning the output from write-only properties.
 		 */
-		$write_only_keys = ['tls_psk_identity', 'tls_psk'];
+		$write_only_keys = ['tls_psk_identity', 'tls_psk', 'name_upper'];
 
 		if ($options['output'] === API_OUTPUT_EXTEND) {
 			$all_keys = array_keys(DB::getSchema($this->tableName())['fields']);
@@ -556,6 +560,7 @@ class CHost extends CHostGeneral {
 
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
+			$result = $this->unsetExtraFields($result, ['name_upper'], $options['output']);
 		}
 
 		// removing keys (hash -> array)
@@ -595,6 +600,17 @@ class CHost extends CHostGeneral {
 	protected function applyQueryOutputOptions($tableName, $tableAlias, array $options, array $sqlParts) {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
+		$upcased_index = array_search($tableAlias.'.name_upper', $sqlParts['select']);
+
+		if ($upcased_index !== false) {
+			unset($sqlParts['select'][$upcased_index]);
+		}
+
+		if (!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output'])) {
+			$sqlParts['select']['inventory_mode'] =
+				dbConditionCoalesce('hinv.inventory_mode', HOST_INVENTORY_DISABLED, 'inventory_mode');
+		}
+
 		if ((!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output']))
 				|| ($options['filter'] && array_key_exists('inventory_mode', $options['filter']))) {
 			$sqlParts['left_join'][] = ['alias' => 'hinv', 'table' => 'host_inventory', 'using' => 'hostid'];
@@ -1479,24 +1495,29 @@ class CHost extends CHostGeneral {
 		}
 
 		// delete the items
-		$del_items = API::Item()->get([
-			'output' => [],
-			'templateids' => $hostids,
-			'nopermissions' => true,
+		$db_items = DB::select('items', [
+			'output' => ['itemid', 'name'],
+			'filter' => [
+				'hostid' => $hostids,
+				'flags' => ZBX_FLAG_DISCOVERY_NORMAL,
+				'type' => CItem::SUPPORTED_ITEM_TYPES
+			],
 			'preservekeys' => true
 		]);
-		if ($del_items) {
-			CItemManager::delete(array_keys($del_items));
+
+		if ($db_items) {
+			CItem::deleteForce($db_items);
 		}
 
-		// delete web tests
-		$delHttptests = [];
-		$dbHttptests = get_httptests_by_hostid($hostids);
-		while ($dbHttptest = DBfetch($dbHttptests)) {
-			$delHttptests[$dbHttptest['httptestid']] = $dbHttptest['httptestid'];
-		}
-		if (!empty($delHttptests)) {
-			API::HttpTest()->delete($delHttptests, true);
+		// delete web scenarios
+		$db_httptests = DB::select('httptest', [
+			'output' => ['httptestid', 'name'],
+			'filter' => ['hostid' => $hostids],
+			'preservekeys' => true
+		]);
+
+		if ($db_httptests) {
+			CHttpTest::deleteForce($db_httptests);
 		}
 
 		// delete host from maps
@@ -1875,7 +1896,7 @@ class CHost extends CHostGeneral {
 	 *
 	 * @param array  $hosts
 	 * @param string $hosts[]['hostid']                    (optional if $db_hosts is null)
-	 * @param int    $hosts[]['tls_connect']               (optionsl)
+	 * @param int    $hosts[]['tls_connect']               (optional)
 	 * @param int    $hosts[]['tls_accept']                (optional)
 	 * @param string $hosts[]['tls_psk_identity']          (optional)
 	 * @param string $hosts[]['tls_psk']                   (optional)
@@ -2523,7 +2544,8 @@ class CHost extends CHostGeneral {
 				'hostids' => zbx_objectValues($hosts, 'hostid'),
 				'skipDependent' => true,
 				'status' => TRIGGER_STATUS_ENABLED,
-				'preservekeys' => true
+				'preservekeys' => true,
+				'nopermissions' => true
 			]);
 
 			$problems = API::Problem()->get([
@@ -2532,7 +2554,8 @@ class CHost extends CHostGeneral {
 				'source' => EVENT_SOURCE_TRIGGERS,
 				'object' => EVENT_OBJECT_TRIGGER,
 				'suppressed' => $options['withProblemsSuppressed'],
-				'severities' => $options['severities']
+				'severities' => $options['severities'],
+				'nopermissions' => true
 			]);
 
 			if (!$problems) {
