@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,8 +31,6 @@ class CTemplateDashboard extends CDashboardGeneral {
 		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
 	];
 
-	protected const AUDIT_RESOURCE = AUDIT_RESOURCE_TEMPLATE_DASHBOARD;
-
 	/**
 	 * @param array $options
 	 *
@@ -45,17 +43,8 @@ class CTemplateDashboard extends CDashboardGeneral {
 			// filter
 			'dashboardids' =>			['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
 			'templateids' =>			['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'default' => null],
-			'filter' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
-				'uuid' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'dashboardid' =>			['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'templateid' =>				['type' => API_IDS, 'flags' => API_ALLOW_NULL | API_NORMALIZE],
-				'display_period' =>			['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => implode(',', DASHBOARD_DISPLAY_PERIODS)],
-				'auto_start' =>				['type' => API_INTS32, 'flags' => API_ALLOW_NULL | API_NORMALIZE, 'in' => '0,1']
-			]],
-			'search' =>					['type' => API_OBJECT, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => [
-				'name' =>					['type' => API_STRINGS_UTF8, 'flags' => API_ALLOW_NULL | API_NORMALIZE]
-			]],
+			'filter' =>					['type' => API_FILTER, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => ['uuid', 'dashboardid', 'name', 'templateid', 'display_period', 'auto_start']],
+			'search' =>					['type' => API_FILTER, 'flags' => API_ALLOW_NULL, 'default' => null, 'fields' => ['name']],
 			'searchByAny' =>			['type' => API_BOOLEAN, 'default' => false],
 			'startSearch' =>			['type' => API_FLAG, 'default' => false],
 			'excludeSearch' =>			['type' => API_FLAG, 'default' => false],
@@ -96,29 +85,82 @@ class CTemplateDashboard extends CDashboardGeneral {
 
 		// permissions
 		if (in_array(self::$userData['type'], [USER_TYPE_ZABBIX_USER, USER_TYPE_ZABBIX_ADMIN])) {
-			if ($options['templateids'] !== null) {
-				$options['templateids'] = array_keys(API::Template()->get([
-					'output' => [],
-					'templateids' => $options['templateids'],
-					'editable' => $options['editable'],
-					'preservekeys' => true
-				]));
+			if ($options['editable']) {
+				if ($options['templateids'] !== null) {
+					$options['templateids'] = array_keys(API::Template()->get([
+						'output' => [],
+						'templateids' => $options['templateids'],
+						'editable' => true,
+						'preservekeys' => true
+					]));
+				}
+				else {
+					$user_groups = getUserGroupsByUserId(self::$userData['userid']);
+
+					$sql_parts['where'][] = 'EXISTS ('.
+						'SELECT NULL'.
+						' FROM hosts_groups hgg'.
+							' JOIN rights r'.
+								' ON r.id=hgg.groupid'.
+									' AND '.dbConditionInt('r.groupid', $user_groups).
+						' WHERE d.templateid=hgg.hostid'.
+						' GROUP BY hgg.hostid'.
+						' HAVING MIN(r.permission)>'.PERM_DENY.
+							' AND MAX(r.permission)>='.PERM_READ_WRITE.
+						')';
+				}
 			}
 			else {
-				$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
 				$user_groups = getUserGroupsByUserId(self::$userData['userid']);
 
-				$sql_parts['where'][] = 'EXISTS ('.
-					'SELECT NULL'.
-					' FROM hosts_groups hgg'.
-						' JOIN rights r'.
-							' ON r.id=hgg.groupid'.
-								' AND '.dbConditionInt('r.groupid', $user_groups).
-					' WHERE d.templateid=hgg.hostid'.
-					' GROUP BY hgg.hostid'.
-					' HAVING MIN(r.permission)>'.PERM_DENY.
-						' AND MAX(r.permission)>='.zbx_dbstr($permission).
-					')';
+				// Select direct templates of all hosts accessible to the current user.
+				$db_host_templates = DBselect(
+					'SELECT DISTINCT ht.templateid FROM hosts_templates ht'.
+					' WHERE ht.hostid IN ('.
+						'SELECT h.hostid FROM hosts h'.
+						' WHERE h.flags IN ('.ZBX_FLAG_DISCOVERY_NORMAL.','.ZBX_FLAG_DISCOVERY_CREATED.')'.
+							' AND h.status IN ('.HOST_STATUS_MONITORED.','.HOST_STATUS_NOT_MONITORED.')'.
+							' AND EXISTS ('.
+								'SELECT NULL'.
+								' FROM hosts_groups hgg'.
+									' JOIN rights r'.
+										' ON r.id=hgg.groupid'.
+											' AND '.dbConditionId('r.groupid', $user_groups).
+								' WHERE h.hostid=hgg.hostid'.
+								' GROUP BY hgg.hostid'.
+								' HAVING MIN(r.permission)>'.PERM_DENY.
+									' AND MAX(r.permission)>='.PERM_READ.
+							')'.
+					')'
+				);
+
+				$templateids = [];
+
+				while ($db_host_template = DBfetch($db_host_templates)) {
+					$templateids[$db_host_template['templateid']] = true;
+				}
+
+				$all_templateids = [];
+
+				while ($templateids) {
+					$all_templateids += $templateids;
+
+					$db_parent_templates = DBselect(
+						'SELECT ht.templateid'.
+						' FROM hosts_templates ht'.
+						' WHERE '.dbConditionId('ht.hostid', array_keys($templateids))
+					);
+
+					$templateids = [];
+
+					while ($db_parent_template = DBfetch($db_parent_templates)) {
+						$templateids[$db_parent_template['templateid']] = true;
+					}
+				}
+
+				$options['templateids'] = $options['templateids'] !== null
+					? array_intersect($options['templateids'], array_keys($all_templateids))
+					: array_keys($all_templateids);
 			}
 		}
 
@@ -203,7 +245,7 @@ class CTemplateDashboard extends CDashboardGeneral {
 
 		$this->updatePages($dashboards);
 
-		$this->addAuditBulk(AUDIT_ACTION_ADD, self::AUDIT_RESOURCE, $dashboards);
+		self::addAuditLog(CAudit::ACTION_ADD, CAudit::RESOURCE_TEMPLATE_DASHBOARD, $dashboards);
 
 		return ['dashboardids' => $dashboardids];
 	}
@@ -235,7 +277,7 @@ class CTemplateDashboard extends CDashboardGeneral {
 
 		$this->updatePages($dashboards, $db_dashboards);
 
-		$this->addAuditBulk(AUDIT_ACTION_UPDATE, self::AUDIT_RESOURCE, $dashboards, $db_dashboards);
+		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_TEMPLATE_DASHBOARD, $dashboards, $db_dashboards);
 
 		return ['dashboardids' => array_column($dashboards, 'dashboardid')];
 	}
@@ -249,7 +291,7 @@ class CTemplateDashboard extends CDashboardGeneral {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['templateid', 'name']], 'fields' => [
 			'uuid' =>			['type' => API_UUID],
 			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('dashboard', 'name')],
-			'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED | API_NOT_EMPTY],
+			'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
 			'display_period' =>	['type' => API_INT32, 'in' => implode(',', DASHBOARD_DISPLAY_PERIODS)],
 			'auto_start' =>		['type' => API_INT32, 'in' => '0,1'],
 			'pages' =>			['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DASHBOARD_MAX_PAGES, 'fields' => [
@@ -291,36 +333,66 @@ class CTemplateDashboard extends CDashboardGeneral {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
 		}
 
-		$this->checkAndAddUuid($dashboards);
+		self::addUuid($dashboards);
+
+		self::checkUuidDuplicates($dashboards);
 		$this->checkDuplicates($dashboards);
 		$this->checkWidgets($dashboards);
 		$this->checkWidgetFields($dashboards);
 	}
 
 	/**
-	 * Check that no duplicate UUID is being added. Add UUID to all template dashboards, if it doesn't exist.
+	 * Add the UUID to those of the given template dashboards that don't have the 'uuid' parameter set.
 	 *
-	 * @param array $dashboards_to_create
-	 *
-	 * @throws APIException
+	 * @param array $dashboards
 	 */
-	protected function checkAndAddUuid(array &$dashboards_to_create): void {
-		foreach ($dashboards_to_create as &$dashboard) {
+	private static function addUuid(array &$dashboards): void {
+		foreach ($dashboards as &$dashboard) {
 			if (!array_key_exists('uuid', $dashboard)) {
 				$dashboard['uuid'] = generateUuidV4();
 			}
 		}
 		unset($dashboard);
+	}
 
-		$db_uuid = DB::select('dashboard', [
+	/**
+	 * Verify template dashboard UUIDs are not repeated.
+	 *
+	 * @param array      $dashboards
+	 * @param array|null $db_dashboards
+	 *
+	 * @throws APIException
+	 */
+	private static function checkUuidDuplicates(array $dashboards, array $db_dashboards = null): void {
+		$dashboard_indexes = [];
+
+		foreach ($dashboards as $i => $dashboard) {
+			if (!array_key_exists('uuid', $dashboard)) {
+				continue;
+			}
+
+			if ($db_dashboards === null || $dashboard['uuid'] !== $db_dashboards[$dashboard['dashboardid']]['uuid']) {
+				$dashboard_indexes[$dashboard['uuid']] = $i;
+			}
+		}
+
+		if (!$dashboard_indexes) {
+			return;
+		}
+
+		$duplicates = DB::select('dashboard', [
 			'output' => ['uuid'],
-			'filter' => ['uuid' => array_column($dashboards_to_create, 'uuid')],
+			'filter' => [
+				'uuid' => array_keys($dashboard_indexes)
+			],
 			'limit' => 1
 		]);
 
-		if ($db_uuid) {
+		if ($duplicates) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Entry with UUID "%1$s" already exists.', $db_uuid[0]['uuid'])
+				_s('Invalid parameter "%1$s": %2$s.', '/'.($dashboard_indexes[$duplicates[0]['uuid']] + 1),
+					_('template dashboard with the same UUID already exists')
+				)
 			);
 		}
 	}
@@ -332,7 +404,8 @@ class CTemplateDashboard extends CDashboardGeneral {
 	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateUpdate(array &$dashboards, array &$db_dashboards = null): void {
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['dashboardid']], 'fields' => [
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['dashboardid']], 'fields' => [
+			'uuid' => 				['type' => API_UUID],
 			'dashboardid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
 			'name' =>				['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('dashboard', 'name')],
 			'display_period' =>		['type' => API_INT32, 'in' => implode(',', DASHBOARD_DISPLAY_PERIODS)],
@@ -368,7 +441,7 @@ class CTemplateDashboard extends CDashboardGeneral {
 		}
 
 		$db_dashboards = $this->get([
-			'output' => ['dashboardid', 'name', 'templateid', 'display_period', 'auto_start'],
+			'output' => ['uuid', 'dashboardid', 'name', 'templateid', 'display_period', 'auto_start'],
 			'dashboardids' => array_column($dashboards, 'dashboardid'),
 			'editable' => true,
 			'preservekeys' => true
@@ -381,7 +454,11 @@ class CTemplateDashboard extends CDashboardGeneral {
 		// Copy original dashboard names and templateids when not specified (for validation and error reporting).
 		$dashboards = $this->extendObjectsByKey($dashboards, $db_dashboards, 'dashboardid', ['name', 'templateid']);
 
-		$api_input_rules = ['type' => API_OBJECTS, 'uniq' => [['name', 'templateid']]];
+		$api_input_rules = ['type' => API_OBJECTS, 'uniq' => [['name', 'templateid']], 'fields' => [
+			'name' =>		['type' => API_STRING_UTF8],
+			'templateid' =>	['type' => API_ID]
+		]];
+
 		if (!CApiInputValidator::validateUniqueness($api_input_rules, $dashboards, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
@@ -392,6 +469,7 @@ class CTemplateDashboard extends CDashboardGeneral {
 		// Check ownership of the referenced pages and widgets.
 		$this->checkReferences($dashboards, $db_dashboards);
 
+		self::checkUuidDuplicates($dashboards, $db_dashboards);
 		$this->checkDuplicates($dashboards, $db_dashboards);
 		$this->checkWidgets($dashboards, $db_dashboards);
 		$this->checkWidgetFields($dashboards, $db_dashboards);
