@@ -1,7 +1,7 @@
 <?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,12 @@ class CFormGrid extends CTag {
 	 */
 	private const ZBX_STYLE_CLASS = 'form-grid';
 
+	public const ZBX_STYLE_FIELDS_GROUP       = 'fields-group';
+	public const ZBX_STYLE_FIELDS_GROUP_LABEL = 'fields-group-label';
+
+	// Adds a margin if contains form actions only.
+	public const ZBX_STYLE_FORM_GRID_ACTIONS  = 'form-grid-actions';
+
 	public const ZBX_STYLE_FORM_GRID_1_1 = 'form-grid-1-1';
 	public const ZBX_STYLE_FORM_GRID_3_1 = 'form-grid-3-1';
 
