@@ -94,11 +94,6 @@ class CDashboardPage extends CBaseComponent {
 
 		this._widgets = new Map();
 
-		this._original_properties = {
-			name: this._data.name,
-			display_period: this._data.display_period
-		};
-
 		this._grid_min_rows = 0;
 		this._grid_pad_rows = 2;
 
@@ -272,17 +267,7 @@ class CDashboardPage extends CBaseComponent {
 	}
 
 	isUnsaved() {
-		if (this._is_unsaved) {
-			return true;
-		}
-
-		for (const [name, value] of Object.entries(this._original_properties)) {
-			if (value != this._data[name]) {
-				return true;
-			}
-		}
-
-		return false;
+		return this._is_unsaved;
 	}
 
 	// Data interface methods.
@@ -343,7 +328,7 @@ class CDashboardPage extends CBaseComponent {
 			});
 		}
 		else {
-			widget = this._createInaccessibleWidget({name, widgetid, pos, unique_id});
+			widget = this._createInaccessibleWidget({widgetid, pos, unique_id});
 		}
 
 		this._doAddWidget(widget);
@@ -408,8 +393,6 @@ class CDashboardPage extends CBaseComponent {
 	}
 
 	replaceWidget(widget, widget_data) {
-		this._is_unsaved = true;
-
 		this.deleteWidget(widget, {is_batch_mode: true});
 
 		return this.addWidget(widget_data);
@@ -446,10 +429,9 @@ class CDashboardPage extends CBaseComponent {
 		});
 	}
 
-	_createInaccessibleWidget({name, widgetid, pos, unique_id}) {
+	_createInaccessibleWidget({widgetid, pos, unique_id}) {
 		return this._createWidget(CWidgetInaccessible, {
 			type: 'inaccessible',
-			name,
 			view_mode: ZBX_WIDGET_VIEW_MODE_HIDDEN_HEADER,
 			fields: {},
 			defaults: {
@@ -725,7 +707,7 @@ class CDashboardPage extends CBaseComponent {
 	}
 
 	_resizeGrid(min_rows = null) {
-		if (min_rows == 0) {
+		if (min_rows === 0) {
 			this._grid_min_rows = 0;
 		}
 		else if (min_rows !== null) {
@@ -734,6 +716,12 @@ class CDashboardPage extends CBaseComponent {
 
 		let num_rows = Math.max(this._grid_min_rows, this._getNumOccupiedRows());
 
+		if (!this._is_edit_mode && num_rows === 0) {
+			this._dashboard_grid.style.height = '';
+
+			return;
+		}
+
 		let height = this._cell_height * num_rows;
 
 		if (this._is_edit_mode) {
