@@ -24,8 +24,9 @@ const ZBX_STYLE_SORTABLE_ITEM = 'sortable-item';
 const ZBX_STYLE_SORTABLE_DRAG_HANDLE = 'sortable-drag-handle';
 const ZBX_STYLE_SORTABLE_DRAGGING = 'sortable-dragging';
 
-const SORTABLE_EVENT_DRAG_START = 'drag_start';
-const SORTABLE_EVENT_DRAG_END = 'drag_end';
+const SORTABLE_EVENT_DRAG_START = 'sortable-drag-start';
+const SORTABLE_EVENT_DRAG_END = 'sortable-drag-end';
+const SORTABLE_EVENT_SORT = 'sortable-sort';
 
 class CSortable extends CBaseComponent {
 
@@ -42,6 +43,7 @@ class CSortable extends CBaseComponent {
 		drag_scroll_delay_short = 150,
 		drag_scroll_delay_long = 400,
 		wheel_step = 100,
+		show_grabbing_cursor = true,
 		do_activate = true
 	}) {
 		super(target);
@@ -51,6 +53,7 @@ class CSortable extends CBaseComponent {
 		this._drag_scroll_delay_short = drag_scroll_delay_short;
 		this._drag_scroll_delay_long = drag_scroll_delay_long;
 		this._wheel_step = wheel_step;
+		this._show_grabbing_cursor = show_grabbing_cursor;
 
 		this._init();
 		this._registerEvents();
@@ -258,6 +261,8 @@ class CSortable extends CBaseComponent {
 
 			item.style.left = `${item_rect.x - list_rect.x}px`;
 			item.style.top = `${item_rect.y - list_rect.y}px`;
+			item.style.width = `${item_rect.width}px`;
+			item.style.height = `${item_rect.height}px`;
 		}
 
 		this._target.classList.add(ZBX_STYLE_SORTABLE_DRAGGING);
@@ -268,11 +273,21 @@ class CSortable extends CBaseComponent {
 		this._drag_item = drag_item;
 		this._drag_item.style.left = `${drag_item_rect.x - target_rect.x}px`;
 		this._drag_item.style.top = `${drag_item_rect.y - target_rect.y}px`;
+		this._drag_item.style.width = `${drag_item_rect.width}px`;
+		this._drag_item.style.height = `${drag_item_rect.height}px`;
+
 		this._target.appendChild(this._drag_item);
 
 		// Hide the actual dragging item.
 		drag_item.classList.add(ZBX_STYLE_SORTABLE_DRAGGING);
 
+		// Set mouse cursor to "grabbing".
+		if (this._show_grabbing_cursor) {
+			this._dragging_style = document.createElement('style');
+			document.head.appendChild(this._dragging_style);
+			this._dragging_style.sheet.insertRule('* { cursor: grabbing !important; }');
+		}
+
 		this.fire(SORTABLE_EVENT_DRAG_START, {item: drag_item});
 	}
 
@@ -348,13 +363,13 @@ class CSortable extends CBaseComponent {
 
 		const drag_item = this._drag_item;
 
-		this._list.insertBefore(drag_item,
-			(this._drag_item_index < items.length) ? items[this._drag_item_index] : null
-		);
+		this._list.insertBefore(drag_item, this._drag_item_index < items.length ? items[this._drag_item_index] : null);
 
 		drag_item.classList.remove(ZBX_STYLE_SORTABLE_DRAGGING);
 		drag_item.style.left = '';
 		drag_item.style.top = '';
+		drag_item.style.width = '';
+		drag_item.style.height = '';
 
 		this._target.classList.remove(ZBX_STYLE_SORTABLE_DRAGGING);
 		this._list.style.width = '';
@@ -363,6 +378,8 @@ class CSortable extends CBaseComponent {
 		for (const item of items) {
 			item.style.left = '';
 			item.style.top = '';
+			item.style.width = '';
+			item.style.height = '';
 		}
 
 		// Re-focus the dragged item.
@@ -370,7 +387,16 @@ class CSortable extends CBaseComponent {
 
 		this._drag_item = null;
 
+		// Reset mouse cursor.
+		if (this._show_grabbing_cursor) {
+			this._dragging_style.remove();
+		}
+
 		this.fire(SORTABLE_EVENT_DRAG_END, {item: drag_item});
+
+		if (this._drag_item_index !== this._drag_item_index_original) {
+			this.fire(SORTABLE_EVENT_SORT);
+		}
 	}
 
 	/**
@@ -472,7 +498,7 @@ class CSortable extends CBaseComponent {
 	 */
 	_cancelDragging() {
 		if (this._drag_item !== null) {
-			// Simulate dropping the item at it's original position.
+			// Simulate dropping the item at its original position.
 
 			this._drag_item_index = this._drag_item_index_original;
 
@@ -773,21 +799,22 @@ class CSortable extends CBaseComponent {
 					return;
 				}
 
-				const reference_item = e.key === 'ArrowLeft'
-					? e.target.previousElementSibling
-					: (e.target.nextElementSibling ? e.target.nextElementSibling.nextElementSibling : null);
-
-				// Leftmost item already focused?
-				if (e.key === 'ArrowLeft' && reference_item === null) {
+				if (e.key === 'ArrowLeft' && e.target.previousElementSibling === null
+						|| e.key === 'ArrowRight' && e.target.nextElementSibling === null) {
 					return;
 				}
 
-				this.insertItemBefore(e.target, reference_item);
+				this.insertItemBefore(e.target, e.key === 'ArrowLeft'
+					? e.target.previousElementSibling
+					: e.target.nextElementSibling.nextElementSibling
+				);
 
 				e.preventDefault();
 
 				// Re-focus the moved item.
 				e.target.focus();
+
+				this.fire(SORTABLE_EVENT_SORT);
 			},
 
 			listFocusIn: (e) => {
