@@ -31,7 +31,7 @@
 		new CWidgetFieldRadioButtonListView($data['fields']['source_type'])
 	)
 	->addField(array_key_exists('sysmapid', $data['fields'])
-		? new CWidgetFieldSelectResourceView($data['fields']['sysmapid'], $data['captions']['simple'])
+		? new CWidgetFieldMultiSelectMapView($data['fields']['sysmapid'], $data['captions']['sysmaps']['sysmapid'])
 		: null
 	)
 	->addField(array_key_exists('filter_widget_reference', $data['fields'])
